#pragma once
#include "hidapi.h"
#include <stdint.h>

#define HID_MAX_TXBUF_SIZE	65
#define HID_MAX_RXBUF_SIZE	64
#define MAX_STR				255
#define MAX_DATA_NUM		160

#define DEVICE_PID	0x003C
#define DEVICE_VID	0x04d8

// Specific setting value
#define COUNT_OF_TUNE_1	610
#define COUNT_OF_TUNE_2	570
#define PICKUP_DEG_1	68
#define PICKUP_DEG_2	36
#define SYS_NUMERATOR_FOR_RPM_1	37500
#define SYS_NUMERATOR_FOR_RPM_2	18750


typedef void (MsgCallback)(char*);

typedef struct _status
{
	uint16_t	rpm;
	uint8_t		sw1;
	uint8_t		e_stop;
	uint8_t		pv_pot;
	uint8_t		pv_target;
	uint8_t		pv_mode;
	uint8_t		reserve1;
	uint16_t	timer1;
	uint8_t		qs_signal;
	uint8_t		qs_state;
} status;

typedef struct _devids
{
	uint16_t userid0;
	uint16_t userid1;
	uint16_t userid2;
	uint16_t userid3;
	uint16_t reserved1;
	uint16_t revid;
	uint16_t devid;
} devids;


typedef struct _IG_DATA
{
	uint8_t	bin[4];
	uint16_t	data[16];
} IG_DATA;

typedef struct _PV_DATA
{
	uint8_t	bin[4];
	uint8_t	data[32];
} PV_DATA;

typedef struct _CONFIG {
	uint8_t	major_version;          // 1
	uint8_t	minor_version;          // 2

	uint8_t	qs_enable_rpm;  		// 3
	uint8_t	qs_sw_on_count;     	// 4
	uint8_t	qs_cut_count;			// 5
	uint8_t	qs_disable_count;		// 6
	uint8_t	qs_use_as_stopsw;      	// 7
	uint8_t	qs_reverse_onoff_state; // 8

	uint8_t	rev_limit;              // 9
	uint8_t	rev_reserved01;         // 10

	uint16_t sys_numerator_for_rpm;	// 11-12
	uint8_t	sys_pulse_per_rotation; // 13
	uint8_t	sys_pickup_degree;     	// 14
	uint8_t sys_reserved00;         // 15
	uint8_t sys_reserved01;         // 16

	uint8_t	dummy[48];              // 17-64
} CONFIG;


typedef union
{
	uint8_t			rxbuf8[64];
	uint16_t		rxbuf16[32];
	status			s;
	devids			d;
	IG_DATA			ig;
	PV_DATA			pv;
	CONFIG			cfg;
} usbPacket;

class HidDevice
{
private:
	uint8_t txbuf[HID_MAX_TXBUF_SIZE];
	hid_device *handle;
	usbPacket rxPacket;


public:
	HidDevice(void);
	~HidDevice(void) {};

	CONFIG cfg = {};

	MsgCallback* msgfunc;

	void callback(MsgCallback* p) {msgfunc=p;}
	void msg( char* p );

	int open(uint16_t vid = DEVICE_VID, uint16_t pid = DEVICE_PID, wchar_t *s_number = 0);
	void close();

	int readTable(double* pIg, double* pPv);
	int writeTable(double* pIg, double* pPv);

	int write( const uint8_t* data = 0, size_t length = 65 );
	int read( uint8_t* data, size_t length = 64 );

	bool isConnected() { return (handle?true:false); }
	bool compareRxTx();
	void HidDevice::checkConfig(void);

	int count_of_tune;					// Adjustment time required for the microcomputer to preprocess
	double count_of_1deg_at_1000rpm;	// Base count time which 1 degree at 1000 rpm. (16Mhz Fosc/4)
	double time2deg(uint16_t src, uint16_t rpm);
	uint16_t deg2time(double src, uint16_t rpm);

};


